# -*- encoding: utf-8 -*-
# --------------------------------------------------------------------------------
# Project:               TransALM
# Copyright:           © by the authors since 2010  All rights reserved.
# --------------------------------------------------------------------------------
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
# --------------------------------------------------------------------------------
# Created:               by telgabsi 
# Last modified:         2010-10-06 08:37:27 PM
#
# Last Author:           $LastChangedBy$
# Last Checkin:          $LastChangedDate$
# Checked out Version:   $LastChangedRevision$
# HeadURL:               $HeadURL$
# --------------------------------------------------------------------------------
from osv import fields, osv
def _serialize_func(orm_obj, vals = {}):
    pickeled = ''
    for attrib in vals:
        if attrib in ('changes', 'modify_date', 'create_date', 'create_uid'):
            continue
        oldval = orm_obj[attrib]
        if type(oldval) == osv.orm.browse_record:
            oldval = orm_obj[attrib].id
        elif type(oldval) == list and len(vals[attrib]) == 0 and len(oldval) == 0:
            continue
        elif type(oldval) == bool:
            oldval = None
        if vals[attrib] != oldval:
            pickeled = pickeled + '<attrib name="%s"><value_old>%s</value_old><value>%s</value></attrib>' % (attrib, oldval, vals[attrib])
    if pickeled == '':
        return None
    return unicode('<xml version="1.0"><%s id="%s">%s</%s>' % (orm_obj._name, orm_obj.id, pickeled, orm_obj._name))

def _deserialize_func(self, cr, uid, ids, field_name, field_value, arg, context):
    '''
    @TODO
    '''
    pass#

def get_previous_change(cr, traceable_obj):
    """
    Finds the previous change
    @return: the id of the last known change or the the passed id
    @param id: the id from where we go back
    @author: telgabsi
    """
    cr.execute('SELECT COALESCE(max(id), 0) FROM %s WHERE traceable = %s' % (traceable_obj._table, traceable_obj.id))
    return cr.fetchone()[0]

class traceable_mixin(object):
    """
    Mixin for traceable objects
    """
    def _create_change_for(self, cr, uid, traceable_obj, vals, context = {}):
        """
        Creates a new change for the requirement
        @param traceable_obj: traceable object to create changes for
        @param change_vals: dictionnary of the value changed
        @return: the id of the newly created record
        @author: telgabsi
        """
        change_vals = {
               'traceable':traceable_obj.id,
               'creation_date':vals['modify_date'],
               'creator': traceable_obj.modifier.id,
               'comment': vals.get('predecessor_id') or _('user %s did not comment this change') % traceable_obj.modifier.id,
               }
        if vals.get('predecessor_id') is not None:
            change_vals['predecessor_id'] = vals['predecessor_id']
        else:
            change_vals['predecessor_id'] = get_previous_change(cr, traceable_obj)
        change_vals['delta'] = traceable._serialize_func(traceable_obj, vals)
        if change_vals['delta'] is not None:
            new_id = osv.osv_pool.get(traceable_obj._name).create(cr, uid, change_vals, context = context)
            return osv.osv_pool.get(traceable_obj._name).browse(cr, uid, new_id, context = context)



class traceable(osv.osv, traceable_mixin):
    '''
    A traceable base entity
    '''
    _auto = False #do not create db table
    _log_access = False # create_date, create_uid, write_uid, write_date
    _name = 'entity.traceable'

    _columns = {
                'creator':  fields.many2one('res.users', 'Creator', readonly = True),
                'creation_date': fields.datetime('Date Created', readonly = True),
                'traceable': fields.integer_big('traced', readonly = True, required = True),
                'comment' : fields.text('Change Log', required = True),
                'predecessor_id': fields.integer('Previous record', required = True, readonly = True),
                #TODO 'delta': fields.serialized('Delta', _serialize_func, _deserialize_func, type = 'binary'),
                'delta': fields.text('Delta', required = True, readonly = True),
                }
    _sql_constraints = [
                        ('predecessor_id', 'UNIQUE (predecessor_id, traceable)', 'predecessor_id must be unique for a traceable entity!'),
                        ]

traceable()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
