# -*- encoding: utf-8 -*-
# --------------------------------------------------------------------------------
# Project:               TransALM
# Copyright:           © by the authors since 2010  All rights reserved.
# --------------------------------------------------------------------------------
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
# --------------------------------------------------------------------------------
# Created                May 6, 2010 by telgabsi
# Last modified:         2010-08-15 09:18:24 AM
#
# Last Author:           $LastChangedBy$
# Last Checkin:          $LastChangedDate$
# Checked out Version:   $LastChangedRevision$
# HeadURL:               $HeadURL$
# --------------------------------------------------------------------------------
from osv import fields, osv
from software_alm.entity.traceable import traceable, traceable_mixin

from tools.translate import _
import time

class requirement_states():
    """
    @author: telgabsi
    @todo: move to a DB-backed entity(inherit from osv.osv) -> customiseable lifecycle
    """
    ACCEPTED = 'accepted'
    DELIVERED = 'delivered'
    DRAFT = 'draft'
    IMPLEMENTATION_ON_HOLD = 'implementation_on_hold'
    IMPLEMENTED = 'implemented'
    IMPLEMENTING = 'implementing'
    IN_ANALYSIS = 'in_analysis'
    IN_SPECIFICATION = 'in_specification'
    IN_TEST = 'in_test'
    NEW = 'new'
    OBSOLETE = 'obsolete'
    REFUSED = 'refused'
    REJECTED = 'rejected'
    RELEASED = 'released'
    SPECIFIED = 'specified'
    TESTED = 'tested'

class software_requirement(osv.osv):
    """
    @author: telgabsi
    The model entity requirement
    """
    _name = 'software.requirement'
    _description = 'Software Requirement'
    _rec_name = 'synopsis'
    _states = [
                (requirement_states.NEW, 'New'),
                (requirement_states.DRAFT, 'Draft'),
                (requirement_states.IN_ANALYSIS, 'In analysis'),
                (requirement_states.REJECTED, 'Rejected'), (requirement_states.OBSOLETE, requirement_states.OBSOLETE),

                (requirement_states.IN_SPECIFICATION, 'In Specification'),
                (requirement_states.SPECIFIED, 'Specified'),

                (requirement_states.IMPLEMENTING, 'Implementing'),
                (requirement_states.IMPLEMENTATION_ON_HOLD, 'Implemention On Hold'),
                (requirement_states.IMPLEMENTED, 'Implemented'),

                (requirement_states.IN_TEST, 'Test running'),
                (requirement_states.TESTED, 'Tested'),

                (requirement_states.RELEASED, 'Released'),
                (requirement_states.DELIVERED, 'Delivered'),
                (requirement_states.ACCEPTED, 'Accepted'), (requirement_states.REFUSED, requirement_states.REFUSED),
               ]
    _columns = {
                'create_date' : fields.datetime('Creation date', readonly = True),
                'modify_date' : fields.datetime('Modification date', readonly = True),
                'modifier': fields.many2one('res.users', 'Last modifier', required = True, readonly = True),
                'owner': fields.many2one('res.users', 'Owner', required = True),
                'synopsis':fields.char('Synopsis', 77, required = True, translate = True),
                'state' : fields.selection(_states, 'Status', required = True, readonly = True),
                #TODO states are externally managed 'state_id': fields.many2many('software.requirement.state', 'res_requirement_state_rel', 'requirement_id', 'state_id', 'State'),
                #TODO'predecessor':
                #TODO 'successor:'
                'description':fields.text('Description', translate = True),
                'features': fields.many2many('software.feature', 'software_feature_requirement_rel', 'requirement_id', 'software_feature_id', 'Features'),
                'project': fields.many2one('project.project', 'Project', ondelete = 'restrict'),
                }
    _defaults = {
        'modifier': lambda self, cr, uid, context: uid,
        'owner': lambda self, cr, uid, context: uid,
        'state': lambda * a: requirement_states.NEW,
    }

    def on_changed_owner(self, cr, uid, ids, owner_id, context = {}):
        """
        @summary: When owner changes make sure that business logic is implemented as required
        @author: telgabsi
        """
        if not owner_id or len(ids) < 1:
            return #no owner passed
        requirement = self.browse(cr, uid, ids[0])
        if not isinstance(requirement, osv.orm.browse_record):
            return #TODO raise error
        if owner_id == requirement.owner.id:
            return #same owner
        elif (self._is_owner_allowed_state(cr, uid, ids, {'state':requirement.state, 'owner': owner_id}, context)):
            return {'value':{'owner': owner_id}}
        else:
            #TODO raise error and reset to previous owner
            return {'value':{'owner': requirement.owner.id}}


    def _get_states_entered_without_tasks(self):
        return (requirement_states.NEW,
                        requirement_states.OBSOLETE, requirement_states.REJECTED,
                        requirement_states.ACCEPTED, requirement_states.REFUSED,
                        )

    def _transition_requires_new_task(self, requirement, newstate, newowner_id):
        """
        @author: telgabsi
        Checks whether the transition from requirement's current state to newstate or/and changing owner to newowner_id
        requires a new @see project.task
        @param requirement: (software.requirement) the requirement in the current state
        @param newstate: (string) the state the requirement is moving to
        @param newowner_id: (long) the new owner of the requirement 
        @return: True or False
        """
        if (newstate and newstate != requirement.state): #state transition
            if (newstate not in self._get_states_entered_without_tasks()):#task needed for new state 
                return True
        #Reaching here means there is no transistion in state or to a state that does not require task

        if (newowner_id and newowner_id != requirement.owner.id):#owner changed
            if (requirement.state not in self._get_states_entered_without_tasks()):#old state requires task
                return True
        return False

    def _get_opentasks(self, requirement):
        """
        @author: telgabsi
        Finds the tasks releated to this requirement that are not cancelled or done.
        @param requirement: (software.requirement) the requirement
        @return: ([string,]) a list of tasks formatted as #task.id!'task.name'
        """
        tasks = []
        for t in requirement.tasks:
            if t.state not in ('done', 'cancelled'): #TODO should be offered by project.task.is_open()
                tasks.append("#%s!'%s'" % (t.id, t.name))
        return tasks

    def _insure_project_tasks(self, cr, uid, ids, vals, context = {}):
        """
        @author: telgabsi
        @summary: enforce workflow
        """
        requirement = self.browse(cr, uid, ids[0], context = context)
        transition_with_task = self._transition_requires_new_task(requirement, vals.get('state'), vals.get('owner'))
        if (transition_with_task):#then close open tasks if any then create a new one
            opentasks = self._get_opentasks(requirement)
            if len(opentasks) > 0:
                raise osv.except_osv(_('Operation not permitted !'), _('Please first close following tasks and retry:\n %s') % ('\n'.join(opentasks)))
            else:
                vals['state'] = vals.get('state') or requirement.state
                vals['owner'] = vals.get('owner') or requirement.owner.id
                task = self._create_task(cr, uid, requirement, vals['state'], vals['owner'], context)
                requirement.tasks.append(task)#Does it make sense?
        return vals

    def _is_owner_allowed_state(self, cr, uid, ids, vals, context = {}):
        """
        @todo approval system
        """
        return True

    def _create_task(self, cr, uid, requirement, state, owner, context = {}):
        """
        Creates a new project.task for the requirement transition
        @param state: the state the during which activities are 
        @param requirement: requierement to create the task for
        @return: the id of the newly created record 
        @author: telgabsi
        """
        if (not requirement.project):
            raise osv.except_osv(_('Error !'), _('Please set the project within which the activities of this requirement will be run'))
        task = {
                'project_id':requirement.project.id,
                'requirement_id':requirement.id,
                'user_id':owner or requirement.owner.id,
                'name':'Task for Requirement #(id:%s)#(state:%s)#(synopsis:%s)' % (requirement.id, state or requirement.state, requirement.synopsis),
                'planned_hours': 8
                }
        new_id = self.pool.get('project.task').create(cr, uid, task, context)
        return self.pool.get('project.task').browse(cr, uid, new_id, context)


    def _deserialize_func(self, cr, uid, ids, field_name, field_value, arg, context):
        return traceable(field_value)

    def write(self, cr, uid, ids, vals, context = {}):
        """
        overriding @see osv.orm.write() to inforce workflow and implement traceability of changes
        """
        vals['modify_date'] = time.strftime('%Y-%m-%d %H:%M:%S')
        vals['modifier'] = uid

        vals = self._insure_project_tasks(cr, uid, ids, vals, context)
        self._record_changes(cr, uid, ids, vals, context)
        return super(software_requirement, self).write(cr, uid, ids, vals, context)

    def _set_state(self, cr, uid, ids, state, context = {}):
        """
        @author: telgabsi
        @todo: check wheter change state is allowed for user (uid == owner || 'requirement_manager' in uid.groups) 
        """
        vals = { 'state' : state }
        self.write(cr, uid, ids, vals, context)

    def requirement_submit(self, cr, uid, ids, context = {}):
        self._set_state(cr, uid, ids, requirement_states.DRAFT, context)
        return True
    def requirement_discard(self, cr, uid, ids, context = {}):
        self._set_state(cr, uid, ids, requirement_states.OBSOLETE, context)
        return True
    def requirement_reject(self, cr, uid, ids, context = {}):
        self._set_state(cr, uid, ids, requirement_states.REJECTED)
        return True
    def requirement_analyse(self, cr, uid, ids, context = {}):
        self._set_state(cr, uid, ids, state = requirement_states.IN_ANALYSIS, context = context)
        return True
    def requirement_spec_order(self, cr, uid, ids, context = {}):
        self._set_state(cr, uid, ids, requirement_states.IN_SPECIFICATION, context)
        return True
    def requirement_spec_release(self, cr, uid, ids, context = {}):
        self._set_state(cr, uid, ids, requirement_states.SPECIFIED, context)
        return True
    def requirement_wait(self, cr, uid, ids, context = {}):
        self._set_state(cr, uid, ids, requirement_states.IMPLEMENTATION_ON_HOLD, context)
        return True
    def requirement_impl_close(self, cr, uid, ids, context = {}):
        self._set_state(cr, uid, ids, requirement_states.IMPLEMENTED, context)
        return True
    def requirement_test(self, cr, uid, ids, context = {}):
        self._set_state(cr, uid, ids, requirement_states.IN_TEST, context)
        return True
    def requirement_test_pass(self, cr, uid, ids, context = {}):
        self._set_state(cr, uid, ids, requirement_states.TESTED, context)
        return True
    def requirement_release(self, cr, uid, ids, context = {}):
        self._set_state(cr, uid, ids, requirement_states.RELEASED, context)
        return True
    def requirement_ship(self, cr, uid, ids, context = {}):
        self._set_state(cr, uid, ids, requirement_states.DELIVERED, context)
        return True
    def requirement_refuse(self, cr, uid, ids, context = {}):
        self._set_state(cr, uid, ids, requirement_states.REFUSED, context)
        return True
    def requirement_accept(self, cr, uid, ids, context = {}):
        self._set_state(cr, uid, ids, requirement_states.ACCEPTED, context)
        return True

software_requirement()

class project_task(osv.osv):
    """
    @author: telgabsi
    @summary: many2many association between project.task and software.requirement
    """
    _inherit = 'project.task'
    _columns = {
                'requirement_id':fields.many2one('software.requirement', required = False, string = 'Requirement', ondelete = 'restrict')
                }
project_task()

class requirement_task(osv.osv):
    """
    @author: telgabsi
    @summary: many2many association between project.task and software.requirement
    """
    _inherit = 'software.requirement'
    _columns = {
                'tasks': fields.one2many('project.task', 'requirement_id', string = 'Tasks', ondelete = 'restrict'),
                }
requirement_task()


#class software_requirement_category(osv.osv):
#    """
#    Categories of requirements
#    """
#    _name = "software.requirement.category"
#    _description = "Category for an software.requirement"
#    _columns = {
#        'name': fields.char('Category', size=64, required=True),
#        'summary': fields.text('Summary'),
#        'parent_id': fields.many2one('software.requirement.category','Parent Categories', ondelete='set null'),
#        'child_ids': fields.one2many('software.requirement.category','parent_id','Child Categories')
#    }
#    _sql_constraints = [
#        ('name', 'unique(parent_id,name)', 'The name of the category must be unique' )
#    ]
#    _order = 'parent_id,name asc'
#software_requirement_category()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
