# -*- encoding: utf-8 -*-
# --------------------------------------------------------------------------------
# Project:               TransALM
# Copyright:           © by the authors since 2010  All rights reserved.
# --------------------------------------------------------------------------------
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
# --------------------------------------------------------------------------------
# Created:               by telgabsi 
# Last modified:         
#
# Last Author:           $LastChangedBy$
# Last Checkin:          $LastChangedDate$
# Checked out Version:   $LastChangedRevision$
# HeadURL:               $HeadURL$
# --------------------------------------------------------------------------------
from osv import fields, osv
from software_alm.entity.traceable import traceable_mixin
from software_alm.entity import traceable
from tools.translate import _

class software_requirement_change(osv.osv, traceable_mixin):
    '''
    @author: telgabsi
    Entity to record changes in a requirment lifecycle
    '''

    _name = 'software.requirement.change'
    _auto = True#explicit because it was off in inherited entity.traceable
    _inherit = 'entity.traceable'
    _columns = {
                'traceable': fields.many2one('software.requirement'),
                }
    _defaults = {
                 }
    def create(self, cr, uid, vals, context = {}):
        """
        overriding osv.osv.create
        """
        requirement = self.pool.get('software.requirement').browse(cr, uid, vals['requirement_id'], context = context)
        del vals['requirement_id']
        #@todo: call traceable_mixin._create_change_for(requirement, vals)
        change_vals = {
               'traceable':requirement.id,
               'creation_date':vals['modify_date'],
               'creator': requirement.modifier.id,
               'comment': vals.get('predecessor_id') or _('user %s did not comment this change') % requirement.modifier.id,
               }
        if vals.get('predecessor_id') is not None:
            change_vals['predecessor_id'] = vals['predecessor_id']
        else:
            cr.execute("SELECT COALESCE(max(id), 0) FROM software_requirement_change WHERE traceable=%s" % requirement.id)
            change_vals['predecessor_id'] = cr.fetchone()[0]
        change_vals['delta'] = traceable._serialize_func(requirement, vals)
        if change_vals['delta'] is not None:
            return super(software_requirement_change, self).create(cr, uid, change_vals, context)

software_requirement_change()

class software_requirement_changes_rel(osv.osv):
    """
    @author: telgabsi
    extending software.requirement with the one2many association to software.requirement.change
    """
    _inherit = 'software.requirement'
    _columns = {
                'changes': fields.one2many('software.requirement.change', 'traceable', 'Changes', readonly = True)
              }
    def _record_changes(self, cr, uid, ids, vals, context = {}):
        """
        Records diff between @param vals: new values from client and persisted values
        @author: telgabsi
        """
        for requirement_id in ids:
            requirement = self.browse(cr, uid, requirement_id, context = context)
            change_id = self._create_software_requirement_change(cr, uid, requirement, vals, context)
            requirement.changes.append(change_id)

    def _create_software_requirement_change(self, cr, uid, requirement, vals, context = {}):
        """
        Creates a new software.requirement.change for the requirement
        @param requirement: requierement to create changes for
        @return: the id of the newly created record 
        @author: telgabsi
        """
        vals['requirement_id'] = requirement.id
        new_id = self.pool.get('software.requirement.change').create(cr, uid, vals, context = context)
        return self.pool.get('software.requirement.change').browse(cr, uid, new_id, context = context)

software_requirement_changes_rel()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: