# -*- encoding: utf-8 -*-
# --------------------------------------------------------------------------------
# Project:               TransALM
# Copyright:           © by the authors since 2010  All rights reserved.
# --------------------------------------------------------------------------------
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
# --------------------------------------------------------------------------------
# Created:               by telgabsi
# Last modified:         2010-08-15 09:18:26 AM
#
# Last Author:           $LastChangedBy$
# Last Checkin:          $LastChangedDate$
# Checked out Version:   $LastChangedRevision$
# HeadURL:               $HeadURL$
# --------------------------------------------------------------------------------
from osv import fields
from osv import osv
import time

class software_test(osv.osv):
    """
    @author: telgabsi
    Entity Software Test
    """
    _name = 'software.test'
    _rec_name = 'name'

    EXECTYPES = [('manual', 'Manual'), ('automated', 'Automated')]

    _columns = {
        'create_date' : fields.datetime('Creation date', readonly=True),
        'modify_date' : fields.datetime('Modification date', readonly=True),
        'modifier': fields.many2one('res.users', 'Last modifier', required=True, readonly=True),
        'name' : fields.char('Name', size=200, required=True),
        'owner': fields.many2one('res.users', 'Owner', required=True),
        'exectype': fields.selection(EXECTYPES, 'Execution Type'),
        'script' : fields.text('Script', translate=True),
        'excecutions': fields.one2many('software.test.execution', 'test_id', string='Executions', readonly=True),
                }
    _defaults = {
        'owner': lambda self, cr, uid, context: uid,
        'modifier': lambda self, cr, uid, context: uid,
        'modify_date' : lambda * t :time.strftime('%Y-%m-%d %H:%M:%S')
                }

    def write(self, cr, uid, ids, vals, context={}):
        """
        overriding @see osv.orm.write() to update modification date and modifier
        """
        vals['modify_date'] = time.strftime('%Y-%m-%d %H:%M:%S')
        vals['modifier'] = uid
        return super(software_test, self).write(cr, uid, ids, vals, context)

    def execute(self, cr, uid, ids, context={}):
        """
        @todo: implement executing the script if exectype=='autmated'
        """
        #tst = self.browse(cr, uid, select, context, list_class, fields_process)
        tst = self.browse(cr, uid, ids[0], context=context)
        if tst.exectype == 'automated':
            #TODO run script
            return
        elif tst.exectype == 'manual':
            #TODO fire add execution dialog
            return

software_test()

class software_test_execution(osv.osv):
    """
    @author: telgabsi
    Execution of test cases
    """
    _name = 'software.test.execution'
    _columns = {
        'execution_datetime' : fields.datetime('Execution date', readonly=True),
        'state': fields.selection([('running', 'Running'), ('failed', 'Failed'), ('succeeded', 'Succeeded')], 'Status'),
        'executer': fields.many2one('res.users', 'Executed by', required=True, readonly=True),
        'log' : fields.text('Log', readonly=True),
        'test_id': fields.many2one('software.test', string='Test', required=True, ondelete='cascade'),
    }
software_test_execution()

class software_requirement(osv.osv):
    """
    @author: telgabsi
    Extending software_requirement with a new attribute tests (many2many)
    """
    _name = 'software.requirement'
    _inherit = 'software.requirement'
    _columns = {
                'tests': fields.many2many('software.test', 'software_test_requirement_rel', 'requirement_id', 'software_test_id', 'Tests'),
               }
software_requirement()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
